using System;
using System.IO;
using System.Text;

namespace Steganography
{
	public class BMPStegoFile : IStegoFile
	{
		
		private string fileName;
		private string password;		
		private string hiddenMessage;
				
		public BMPStegoFile(string fileName, string password) {
			this.fileName = fileName;
			this.password = password;
		}
		
		// Return the hidden message
		public string HiddenMessage {
			get {
					if (hiddenMessage==null)
						ExtractHiddenMessage();						
					return hiddenMessage;
			}
		}
		
		protected void ExtractHiddenMessage() {
			
			// Open the bmp file
			Stream inStream = new FileStream(fileName, FileMode.Open, FileAccess.Read);		
			
			// Data starts at byte 54
			int offset = 54;						
			inStream.Seek(offset, 0);							
			
			// Decode the first 4 bytes which contain
			// the length of the message, using the LSB method
			byte[] bLen = LSBHelper.Decode(inStream, 4);					
			
			// Decript the length
			bLen = CryptoHelper.Decrypt(bLen, password);			
			int len = BitConverter.ToInt32(bLen, 0);

			// Decode the message using the LSB method
			inStream.Seek(offset + 4*8 , 0);
	
			byte[] buffer;

			try 
			{
				buffer = LSBHelper.Decode(inStream, len);									
			} catch {
				throw new Exception("Wrong password or not a stego file!");
			}
			
			// Add the length ahead (4 bytes)
			buffer = ConcatByteArray(bLen, buffer);						

			// Decript the message (including the first 4 bytes)
			buffer = CryptoHelper.Decrypt(buffer, password);

			// Extract only the message (without the first 4 bytes)
			byte[] byteMessage = GetByteMessage(buffer);			

			// Get the hidden unicode message
			UnicodeEncoding unicode = new UnicodeEncoding();								
			hiddenMessage = unicode.GetString(byteMessage);

			inStream.Close();			
		
		}
		
		// Concatenate two byte arrays
		private byte[] ConcatByteArray(byte[] a, byte[] b) {
			byte[] ret = new byte[a.Length + b.Length];
			for (int i=0; i < a.Length; i++)
				ret[i] = a[i];
			for (int i=0; i < b.Length; i++)
				ret[i+a.Length] = b[i];
			return ret;
		}
		
		// Return a the byte array in input without
		// the first 4 bytes which represent the
		// length of the message
		private byte[] GetByteMessage(byte[] buffer) {
			byte[] ret = new byte[buffer.Length-4];
			for (int i=4; i < buffer.Length; i++)
				ret[i-4] = buffer[i];			
			return ret;
		}
	}	
}
