using System;
using System.IO;
using System.Text;

namespace Steganography
{
	public class BMPCoverFile : ICoverFile
	{
		
		private string fileName;
				
		public BMPCoverFile(string fileName) {
			this.fileName = fileName;
		}
		
		public IStegoFile CreateStegoFile(string stegoFileName, string message, string password) {
			
			// Open the cover
			FileStream inStream = new FileStream(fileName, FileMode.Open, FileAccess.Read);
			
			// Check whether the cover is a bmp file
			char b = (char) inStream.ReadByte();
			char m = (char) inStream.ReadByte();			
			if (!(b=='B' && m=='M'))
				throw new Exception("The file is not a bitmap!");
				
			// Check whether the bitmap is 24bit
			byte[] buffer = new byte[2];
			inStream.Seek(28, 0);
			inStream.Read(buffer,0,2);			
			Int16 nBit = BitConverter.ToInt16(buffer, 0);						
			if (nBit!=24)
				throw new Exception("The file is not a 24bit bitmap!");
			
			// Read the header from the cover			
			int offset = 54;	// Header Size		
			byte[] header = new byte[offset];
			inStream.Seek(0, 0);
			inStream.Read(header, 0, offset);
			
			// Write the header to the stego file
			FileStream outStream = new FileStream(stegoFileName, FileMode.Create, FileAccess.Write);

			outStream.Write(header, 0, offset);
			
			// Create the unicode encoding
			UnicodeEncoding unicode = new UnicodeEncoding();												
			
			// Add the length of the message inside the first 4 bytes
			byte[] messageBytes = AddLengthAhead(unicode.GetBytes(message));      
      
			// Encrypt the message
			messageBytes = CryptoHelper.Encrypt(messageBytes, password);
      
			// Encode the cover using the LSB method
     		inStream.Seek(offset, 0);			     	
     		LSBHelper.Encode(inStream, messageBytes, outStream);     	     	
     	     	
			inStream.Close();
			outStream.Close();
			
			// Return a new BMPStegoFile
			return new BMPStegoFile(stegoFileName, password);
			
		}
		
		private byte[] AddLengthAhead(byte[] messageBytes) {
			int len = messageBytes.Length;
			byte[] bLen = BitConverter.GetBytes(len);
			byte[] ret = new byte[len + bLen.Length];
			for (int i=0; i < bLen.Length; i++)
				ret[i] = bLen[i];
			for (int i=0; i < messageBytes.Length; i++)
				ret[i+bLen.Length] = messageBytes[i];
			return ret;
		}
		
	}
	
}
